@@ -17,21 +17,23 @@
 
 import adb
 import argparse
+import json
 import logging
 import os
 import re
 import subprocess
 import sys
+import textwrap
 
 # Shared functions across gdbclient.py and ndk-gdb.py.
 import gdbrunner
 
 def get_gdbserver_path(root, arch):
-    path = "{}/prebuilts/misc/android-{}/gdbserver{}/gdbserver{}"
+    path = "{}/prebuilts/misc/gdbserver/android-{}/gdbserver{}"
     if arch.endswith("64"):
-        return path.format(root, arch, "64", "64")
+        return path.format(root, arch, "64")
     else:
-        return path.format(root, arch, "", "")
+        return path.format(root, arch, "")
 
 
 def get_tracer_pid(device, pid):
@@ -64,32 +66,25 @@ def parse_args():
     parser.add_argument(
         "--user", nargs="?", default="root",
         help="user to run commands as on the device [default: root]")
+    parser.add_argument(
+        "--setup-forwarding", default=None, choices=["gdb", "vscode"],
+        help=("Setup the gdbserver and port forwarding. Prints commands or " +
+              ".vscode/launch.json configuration needed to connect the debugging " +
+              "client to the server."))
+
+    parser.add_argument(
+        "--env", nargs=1, action="append", metavar="VAR=VALUE",
+        help="set environment variable when running a binary")
 
     return parser.parse_args()
 
 
-def dump_var(root, variable):
-    make_args = ["make", "CALLED_FROM_SETUP=true",
-                 "BUILD_SYSTEM={}/build/core".format(root),
-                 "--no-print-directory", "-f",
-                 "{}/build/core/config.mk".format(root),
-                 "dumpvar-{}".format(variable)]
-
-    # subprocess cwd argument does not change the PWD shell variable, but
-    # dumpvar.mk uses PWD to create an absolute path, so we need to set it.
-    saved_pwd = os.environ['PWD']
-    os.environ['PWD'] = root
-    make_output = subprocess.check_output(make_args, cwd=root)
-    os.environ['PWD'] = saved_pwd
-    return make_output.splitlines()[0]
-
-
 def verify_device(root, device):
-    names = set([device.get_prop("ro.build.product"), device.get_prop("ro.product.device")])
-    target_device = dump_var(root, "TARGET_DEVICE")
-    if target_device not in names:
-        msg = "TARGET_DEVICE ({}) does not match attached device ({})"
-        sys.exit(msg.format(target_device, ", ".join(names)))
+    name = device.get_prop("ro.product.name")
+    target_device = os.environ["TARGET_PRODUCT"]
+    if target_device != name:
+        msg = "TARGET_PRODUCT ({}) does not match attached device ({})"
+        sys.exit(msg.format(target_device, name))
 
 
 def get_remote_pid(device, process_name):
@@ -169,32 +164,60 @@ def handle_switches(args, sysroot):
 
     return (binary_file, pid, run_cmd)
 
-def generate_gdb_script(sysroot, binary_file, is64bit, port, connect_timeout=5):
-    # Generate a gdb script.
-    # TODO: Detect the zygote and run 'art-on' automatically.
-    root = os.environ["ANDROID_BUILD_TOP"]
-    symbols_dir = os.path.join(sysroot, "system", "lib64" if is64bit else "lib")
-    vendor_dir = os.path.join(sysroot, "vendor", "lib64" if is64bit else "lib")
+def generate_vscode_script(gdbpath, root, sysroot, binary_name, port, dalvik_gdb_script, solib_search_path):
+    # TODO It would be nice if we didn't need to copy this or run the
+    #      gdbclient.py program manually. Doing this would probably require
+    #      writing a vscode extension or modifying an existing one.
+    res = {
+        "name": "(gdbclient.py) Attach {} (port: {})".format(binary_name.split("/")[-1], port),
+        "type": "cppdbg",
+        "request": "launch",  # Needed for gdbserver.
+        "cwd": root,
+        "program": binary_name,
+        "MIMode": "gdb",
+        "miDebuggerServerAddress": "localhost:{}".format(port),
+        "miDebuggerPath": gdbpath,
+        "setupCommands": [
+            {
+                # Required for vscode.
+                "description": "Enable pretty-printing for gdb",
+                "text": "-enable-pretty-printing",
+                "ignoreFailures": True,
+            },
+            {
+                "description": "gdb command: dir",
+                "text": "-environment-directory {}".format(root),
+                "ignoreFailures": False
+            },
+            {
+                "description": "gdb command: set solib-search-path",
+                "text": "-gdb-set solib-search-path {}".format(":".join(solib_search_path)),
+                "ignoreFailures": False
+            },
+            {
+                "description": "gdb command: set solib-absolute-prefix",
+                "text": "-gdb-set solib-absolute-prefix {}".format(sysroot),
+                "ignoreFailures": False
+            },
+        ]
+    }
+    if dalvik_gdb_script:
+        res["setupCommands"].append({
+            "description": "gdb command: source art commands",
+            "text": "-interpreter-exec console \"source {}\"".format(dalvik_gdb_script),
+            "ignoreFailures": False,
+        })
+    return json.dumps(res, indent=4)
 
-    solib_search_path = []
-    symbols_paths = ["", "hw", "ssl/engines", "drm", "egl", "soundfx"]
-    vendor_paths = ["", "hw", "egl"]
-    solib_search_path += [os.path.join(symbols_dir, x) for x in symbols_paths]
-    solib_search_path += [os.path.join(vendor_dir, x) for x in vendor_paths]
+def generate_gdb_script(root, sysroot, binary_name, port, dalvik_gdb_script, solib_search_path, connect_timeout):
     solib_search_path = ":".join(solib_search_path)
 
     gdb_commands = ""
-    gdb_commands += "file '{}'\n".format(binary_file.name)
+    gdb_commands += "file '{}'\n".format(binary_name)
     gdb_commands += "directory '{}'\n".format(root)
     gdb_commands += "set solib-absolute-prefix {}\n".format(sysroot)
     gdb_commands += "set solib-search-path {}\n".format(solib_search_path)
-
-    dalvik_gdb_script = os.path.join(root, "development", "scripts", "gdb",
-                                     "dalvik.gdb")
-    if not os.path.exists(dalvik_gdb_script):
-        logging.warning(("couldn't find {} - ART debugging options will not " +
-                         "be available").format(dalvik_gdb_script))
-    else:
+    if dalvik_gdb_script:
         gdb_commands += "source {}\n".format(dalvik_gdb_script)
 
     # Try to connect for a few seconds, sometimes the device gdbserver takes
@@ -207,7 +230,7 @@ def target_remote_with_retry(target, timeout_seconds):
   end_time = time.time() + timeout_seconds
   while True:
     try:
-      gdb.execute("target remote " + target)
+      gdb.execute("target extended-remote " + target)
       return True
     except gdb.error as e:
       time_left = end_time - time.time()
@@ -224,8 +247,42 @@ end
 
     return gdb_commands
 
+def generate_setup_script(gdbpath, sysroot, binary_file, is64bit, port, debugger, connect_timeout=5):
+    # Generate a setup script.
+    # TODO: Detect the zygote and run 'art-on' automatically.
+    root = os.environ["ANDROID_BUILD_TOP"]
+    symbols_dir = os.path.join(sysroot, "system", "lib64" if is64bit else "lib")
+    vendor_dir = os.path.join(sysroot, "vendor", "lib64" if is64bit else "lib")
+
+    solib_search_path = []
+    symbols_paths = ["", "hw", "ssl/engines", "drm", "egl", "soundfx"]
+    vendor_paths = ["", "hw", "egl"]
+    solib_search_path += [os.path.join(symbols_dir, x) for x in symbols_paths]
+    solib_search_path += [os.path.join(vendor_dir, x) for x in vendor_paths]
+
+    dalvik_gdb_script = os.path.join(root, "development", "scripts", "gdb", "dalvik.gdb")
+    if not os.path.exists(dalvik_gdb_script):
+        logging.warning(("couldn't find {} - ART debugging options will not " +
+                         "be available").format(dalvik_gdb_script))
+        dalvik_gdb_script = None
+
+    if debugger == "vscode":
+        return generate_vscode_script(
+            gdbpath, root, sysroot, binary_file.name, port, dalvik_gdb_script, solib_search_path)
+    elif debugger == "gdb":
+        return generate_gdb_script(root, sysroot, binary_file.name, port, dalvik_gdb_script, solib_search_path, connect_timeout)
+    else:
+        raise Exception("Unknown debugger type " + debugger)
+
 
 def main():
+    required_env = ["ANDROID_BUILD_TOP",
+                    "ANDROID_PRODUCT_OUT", "TARGET_PRODUCT"]
+    for env in required_env:
+        if env not in os.environ:
+            sys.exit(
+                "Environment variable '{}' not defined, have you run lunch?".format(env))
+
     args = parse_args()
     device = args.device
 
@@ -233,7 +290,7 @@ def main():
         sys.exit("ERROR: Failed to find device.")
 
     root = os.environ["ANDROID_BUILD_TOP"]
-    sysroot = dump_var(root, "abs-TARGET_OUT_UNSTRIPPED")
+    sysroot = os.path.join(os.environ["ANDROID_PRODUCT_OUT"], "symbols")
 
     # Make sure the environment matches the attached device.
     verify_device(root, device)
@@ -254,24 +311,22 @@ def main():
 
         tracer_pid = get_tracer_pid(device, pid)
         if tracer_pid == 0:
+            cmd_prefix = args.su_cmd
+            if args.env:
+                cmd_prefix += ['env'] + [v[0] for v in args.env]
+
             # Start gdbserver.
             gdbserver_local_path = get_gdbserver_path(root, arch)
             gdbserver_remote_path = "/data/local/tmp/{}-gdbserver".format(arch)
             gdbrunner.start_gdbserver(
                 device, gdbserver_local_path, gdbserver_remote_path,
                 target_pid=pid, run_cmd=run_cmd, debug_socket=debug_socket,
-                port=args.port, run_as_cmd=args.su_cmd)
+                port=args.port, run_as_cmd=cmd_prefix)
         else:
             print "Connecting to tracing pid {} using local port {}".format(tracer_pid, args.port)
             gdbrunner.forward_gdbserver_port(device, local=args.port,
                                              remote="tcp:{}".format(args.port))
 
-        # Generate a gdb script.
-        gdb_commands = generate_gdb_script(sysroot=sysroot,
-                                           binary_file=binary_file,
-                                           is64bit=is64bit,
-                                           port=args.port)
-
         # Find where gdb is
         if sys.platform.startswith("linux"):
             platform_name = "linux-x86"
@@ -279,14 +334,39 @@ def main():
             platform_name = "darwin-x86"
         else:
             sys.exit("Unknown platform: {}".format(sys.platform))
+
         gdb_path = os.path.join(root, "prebuilts", "gdb", platform_name, "bin",
                                 "gdb")
+        # Generate a gdb script.
+        setup_commands = generate_setup_script(gdbpath=gdb_path,
+                                               sysroot=sysroot,
+                                               binary_file=binary_file,
+                                               is64bit=is64bit,
+                                               port=args.port,
+                                               debugger=args.setup_forwarding or "gdb")
 
-        # Print a newline to separate our messages from the GDB session.
-        print("")
+        if not args.setup_forwarding:
+            # Print a newline to separate our messages from the GDB session.
+            print("")
 
-        # Start gdb.
-        gdbrunner.start_gdb(gdb_path, gdb_commands)
+            # Start gdb.
+            gdbrunner.start_gdb(gdb_path, setup_commands)
+        else:
+            print("")
+            print setup_commands
+            print("")
+            if args.setup_forwarding == "vscode":
+                print textwrap.dedent("""
+                        Paste the above json into .vscode/launch.json and start the debugger as
+                        normal. Press enter in this terminal once debugging is finished to shutdown
+                        the gdbserver and close all the ports.""")
+            else:
+                print textwrap.dedent("""
+                        Paste the above gdb commands into the gdb frontend to setup the gdbserver
+                        connection. Press enter in this terminal once debugging is finished to
+                        shutdown the gdbserver and close all the ports.""")
+            print("")
+            raw_input("Press enter to shutdown gdbserver")
 
 if __name__ == "__main__":
     main()
