@@ -4,22 +4,24 @@ import os
 import re
 import sys
 
-from utils import run_header_checker
+import_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
+import_path = os.path.abspath(os.path.join(import_path, 'utils'))
+sys.path.insert(1, import_path)
 
-SCRIPT_DIR = os.path.abspath(os.path.dirname(__file__))
-INPUT_DIR = os.path.join(SCRIPT_DIR, 'input')
-EXPECTED_DIR = os.path.join(SCRIPT_DIR, 'expected')
-
-DEFAULT_CFLAGS = ['-x', 'c++', '-std=c++11']
+from utils import run_header_abi_dumper
+from module import Module
+from test import INPUT_DIR
+from test import EXPECTED_DIR
+from test import make_and_copy_reference_dumps
 
 FILE_EXTENSIONS = ['h', 'hpp', 'hxx', 'cpp', 'cc', 'c']
 
+
 def main():
     patt = re.compile(
-        '^.*\\.(?:' + \
-        '|'.join('(?:' + re.escape(ext) + ')' for ext in FILE_EXTENSIONS) + \
+        '^.*\\.(?:' +
+        '|'.join('(?:' + re.escape(ext) + ')' for ext in FILE_EXTENSIONS) +
         ')$')
-
     input_dir_prefix_len = len(INPUT_DIR) + 1
     for base, dirnames, filenames in os.walk(INPUT_DIR):
         for filename in filenames:
@@ -32,12 +34,17 @@ def main():
             output_path = os.path.join(EXPECTED_DIR, input_rel_path)
 
             print('generating', output_path, '...')
-            output_content = run_header_checker(input_path, DEFAULT_CFLAGS)
+            output_content = run_header_abi_dumper(input_path)
 
             os.makedirs(os.path.dirname(output_path), exist_ok=True)
             with open(output_path, 'w') as f:
                 f.write(output_content)
+    modules = Module.get_test_modules()
+    for module in modules:
+        print('Created abi dump at', make_and_copy_reference_dumps(module))
+
     return 0
 
+
 if __name__ == '__main__':
     sys.exit(main())
